'use strict';
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadModel = exports.get_all_words_list = exports.save_all_words_list = exports.processPredictData = exports.TextFeatures = exports.TextProcessing = void 0;
const fs_1 = __importDefault(require("fs"));
const readline_1 = __importDefault(require("readline"));
const lodash_1 = require("lodash");
const boa = require('@pipcook/boa');
const jieba = boa.import('jieba');
const random = boa.import('random');
const pickle = boa.import('pickle');
const { open, list, set } = boa.builtins();
function strip(str) {
    return str.replace(/(^\s*)|(\s*$)/g, '');
}
function isdigit(str) {
    return /^\d+$/.test(str);
}
function MakeWordsSet(words_file) {
    const words_set = new Set();
    const rl = readline_1.default.createInterface({
        input: fs_1.default.createReadStream(words_file)
    });
    return new Promise((resolve) => {
        rl.on('line', (line) => {
            const word = strip(line);
            if (word.length > 0 && !words_set.has(word)) {
                words_set.add(word);
            }
        });
        rl.on('close', () => {
            resolve(words_set);
        });
    });
}
exports.TextProcessing = function (row_data, row_class) {
    const data_list = [];
    const class_list = [];
    row_data.forEach((data, i) => {
        const word_cut = jieba.cut(data, boa.kwargs({
            cut_all: false
        }));
        data_list.push(list(word_cut));
        class_list.push(row_class[i]);
    });
    // split train set and testing set
    const data_class_list = lodash_1.zip(data_list, class_list);
    random.shuffle(data_class_list);
    const [train_data_list, train_class_list] = lodash_1.unzip(data_class_list);
    const all_words_dict = {};
    for (const word_list of train_data_list) {
        for (const word of word_list) {
            if (!all_words_dict[word]) {
                all_words_dict[word] = [word, 1];
            }
            else {
                all_words_dict[word][1] += 1;
            }
        }
    }
    const all_words_tuple_list = Object.values(all_words_dict).sort((item1, item2) => item2[1] - item1[1]);
    const all_words_list = lodash_1.unzip(all_words_tuple_list)[0];
    return [all_words_list, train_data_list, train_class_list];
};
function words_dict(all_words_list, stopwords_set = new Set()) {
    const feature_words = [];
    for (let word of all_words_list) {
        if (!isdigit(word) &&
            !stopwords_set.has(word) &&
            word.length > 1 &&
            word.length < 5) {
            feature_words.push(word);
        }
    }
    return feature_words;
}
function text_features(text, feature_words) {
    const text_words = set(text);
    return boa.eval `[1 if word in ${text_words} else 0 for word in ${feature_words}]`;
}
exports.TextFeatures = function (train_data_list, feature_words) {
    return train_data_list.map((text) => {
        return text_features(text, feature_words);
    });
};
exports.processPredictData = function (data, all_words_list_path, stopwords_path) {
    const all_words_list = pickle.load(open(all_words_list_path, 'rb'));
    const word_cut = jieba.cut(data, boa.kwargs({
        cut_all: false
    }));
    const stopwords_file = stopwords_path;
    return MakeWordsSet(stopwords_file).then((stopwords_set) => {
        const feature_words = words_dict(all_words_list, stopwords_set);
        return list(word_cut).map((text) => {
            return text_features(text, feature_words);
        });
    });
};
exports.save_all_words_list = function (feature_words, filepath) {
    pickle.dump(feature_words, open(filepath, 'wb'));
};
exports.get_all_words_list = function (filepath) {
    return __awaiter(this, void 0, void 0, function* () {
        return boa.with(open(filepath, 'rb'), pickle.load);
    });
};
exports.loadModel = function (filepath) {
    return __awaiter(this, void 0, void 0, function* () {
        return boa.with(open(filepath, 'rb'), (f) => {
            const m = pickle.load(f);
            return m;
        });
    });
};
//# sourceMappingURL=script.js.map